import argparse
import logging

from .app import (AppException, DataFrameManipulation, file_validation,
                  open_spark_session)

main_logger = logging.getLogger(__name__)


def main():
    try:
        parser = argparse.ArgumentParser()
        parser.add_argument("-sp", "--sparkremoteurl", required=True)
        parser.add_argument("-cli", "--clientsdatasetpath", required=True)
        parser.add_argument("-fin", "--finDetailsdatasetpath", required=True)
        parser.add_argument("-c", "--country", action="append", required=True)
        args = parser.parse_args()
        file_validation(filePath=args.clientsdatasetpath)
        file_validation(filePath=args.finDetailsdatasetpath)

        with open_spark_session(spark_remote_url=args.sparkremoteurl) as spark:
            dfm = DataFrameManipulation()
            dfm.clients = spark.read.csv(args.clientsdatasetpath,
                                         sep=',',
                                         header=True)
            dfm.finDetails = spark.read.csv(args.finDetailsdatasetpath,
                                            sep=',',
                                            header=True)
            dfm.clients.show()
            dfm.finDetails.show()
            dfm.filter_rows(filterConditions={"country": args.country})
            dfm.select_columns(colsList=['email', 'country'],
                               colsMap={"btc_a": "bitcoin_address",
                                        "id": "client_identifier",
                                        "cc_t": "credit_card_type"})
            dfm.clients.show()
            dfm.save_output(outputPath="client_data")
    except AppException as e:
        main_logger.error(e)
    except Exception:
        main_logger.info("Programm completed with an ERROR!")


if __name__ == "__main__":
    main()



import pyspark
  
# importing sparksession from 
# pyspark.sql module
from pyspark.sql import SparkSession

from pyspark.sql.types import StringType, StructField

from collections import OrderedDict
from csv import DictReader

StructType([StructField('client_identifier', StringType(), True),
            StructField('email', StringType(), True),
            StructField('country', StringType(), True),
            StructField('bitcoin_address', StringType(), True),
            StructField('credit_card_type', StringType(), True)])



# open file in read mode
#with open("source_data/dataset_one.csv", 'r') as f:
#      for line in DictReader(f):
#            print(line)

SparkSession.createDataFrame()

# creating sparksession and giving 
# an app name
#spark = SparkSession.builder.appName('sparkdf').getOrCreate()
spark = SparkSession.builder.remote("sc://localhost:15002").getOrCreate()
  
# list  of college data with  dictionary
data = [OrderedDict([('id', '10'), ('first_name', 'Friedrich'), ('last_name', 'Kreutzer'), ('email', 'fkreutzer9@businessweek.com'), ('country', 'France')]),
OrderedDict([('id', '11'), ('first_name', 'Conn'), ('last_name', 'Claiden'), ('email', 'cclaidena@vimeo.com'), ('country', 'France')])]
  
# creating a dataframe
dataframe = spark.createDataFrame(data)
  
# show data frame
dataframe.show()
"""
from pyspark.sql import SparkSession
from pyspark.sql.dataframe import DataFrame
from pyspark.sql.functions import col
from typing import Dict, Union, Optional, List


clients: DataFrame
finDetails: DataFrame


##################################
#spark = SparkSession.builder.remote("sc://localhost:15002").getOrCreate()
spark = SparkSession.builder.master("local").appName("chispa").getOrCreate()
clients = spark.read.csv('C:\\Users\\mykkis\\CapGeminiProjects\\userStory8020\\source_data\\dataset_one.csv', sep=',', header=True).alias("clients")
#finDetails = spark.read.csv('/usr/src/userStory8020/source_data/dataset_two.csv', sep=',', header=True).alias("finDetails")
#clients = clients.join(finDetails, ["id"])
clients.show()
clients.write.mode("overwrite").option("header",True).csv("C:\\Users\\mykkis\\CapGeminiProjects\\userStory8020\\outputcsv")


from csv import DictReader


    # open file in read mode
with open("source_data/dataset_one.csv", 'r') as f:
      dict_reader = DictReader(f)
      list_of_dict = list(dict_reader)
      print(list_of_dict)

clients = spark.createDataFrame([{'student_id': 12, 'name': 'sravan', 'address': 'kakumanu'},
                                 {'student_id': 14, 'name': 'jyothika', 'address': 'tenali'},
                                 {'student_id': 11, 'name': 'deepika', 'address': 'repalle'}])
clients.show()




clients = filter_rows(df=clients,
                      filterConditions={"country": ["United Kingdom",
                                                    "Netherlands"]})
clients.show()


clients = select_columns(df=clients,
                         colsList=['email', 'country'],
                         colsMap={"btc_a": "bitcoin_address",
                                  "id": "client_identifier",
                                  "cc_t": "credit_card_type"})
clients.show()




finDetails = spark.read.csv('/usr/src/userStory8020/source_data/dataset_two.csv', sep=',', header=True).alias("finDetails")
clients.show()
finDetails.show()

#clients = clients.where(clients.country.isin("United Kingdom", "Netherlands"))
#clients.show()
clients = clients.join(finDetails, (clients.id == finDetails.id)) \
                 .select(col("clients.id").alias("id") \
                       , clients.email \
                       , clients.country \
                       , col("finDetails.btc_a").alias("bitcoin_address") \
                       , col("finDetails.cc_t").alias("credit_card_type")) \
                 .where(clients.country.isin("United Kingdom", "Netherlands"))
clients.show()
#clients.write.mode("overwrite").option("header",True).csv("/usr/src/userStory8020/client_data/outputcsv")
"""


import logging
from contextlib import contextmanager
from csv import Error, Sniffer
from os import R_OK, access
from pathlib import Path
from typing import Dict, List, Optional, Union

from pyspark.sql import SparkSession
from pyspark.sql.connect.dataframe import DataFrame
from pyspark.sql.functions import col

app_logger = logging.getLogger(__name__)


@contextmanager
def open_spark_session(spark_remote_url: str):
    spark_session = SparkSession.builder \
                                .remote(spark_remote_url) \
                                .getOrCreate()
    try:
        app_logger.info("Spark session is opened.")
        yield spark_session
    finally:
        spark_session.stop()
        app_logger.info("Spark session is closed.")


class DataFrameManipulation:
    def __init__(self) -> None:
        self.__clients: DataFrame
        self.__finDetails: DataFrame

    @property
    def clients(self) -> DataFrame:
        return self.__clients

    @clients.setter
    def clients(self, df: DataFrame) -> None:
        if isinstance(df, DataFrame):
            self.__clients = df

    @property
    def finDetails(self):
        return self.__finDetails

    @finDetails.setter
    def finDetails(self, df: DataFrame) -> None:
        if isinstance(df, DataFrame):
            self.__finDetails = df

    def filter_rows(self,
                    filterConditions: Dict[str,
                                           Union[str,
                                                 List[str]]]) -> None:
        logging.info('Filtering data.')
        logging.debug(f'filterConditions - {filterConditions}')
        for col_name, col_value in filterConditions.items():
            self.clients = self.clients \
                               .filter(self.clients[col_name].isin(col_value))
        logging.info('Data is filtered.')

    def select_columns(self,
                       colsList: List[str],
                       colsMap: Optional[Dict[str, str]] = None) -> None:
        logging.info('Selecting columns.')
        logging.debug(f'colsList - {colsList}')
        self.clients = self.clients.join(self.finDetails, ["id"])
        if colsMap:
            self.rename_columns(colsMap=colsMap)
            colsList.extend(colsMap.values())
        self.clients = self.clients.select([c for c in self.clients.columns if c in colsList])
        logging.info('Columns selected.')

    def rename_columns(self,
                       colsMap: Dict[str, str]) -> None:
        logging.info('Renaming columns.')
        logging.debug(f'colsMap - {colsMap}')
        self.clients = self.clients.select([col(c).alias(colsMap.get(c, c)) for c in self.clients.columns])
        logging.info('Columns renamed.')

    def save_output(self, outputPath: str):
        logging.info('Saving output.')
        logging.debug(f'OutputPath - {outputPath}')
        self.clients.write \
                    .mode("overwrite") \
                    .option("header", True) \
                    .csv(outputPath)
        logging.info('Output saved.')


def file_validation(filePath: str) -> None:
    p = Path(filePath)
    if p.exists() and p.is_file():
        if access(filePath, R_OK):
            try:
                with open(filePath, newline='') as csvfile:
                    start = csvfile.read(1024)
                    Sniffer().sniff(start)
                    if not Sniffer().has_header(start):
                        app_logger.error("Seems to be the header is absent.")
                        raise AppException(f'file: {filePath} is incorrect!')
            except Error as e:
                app_logger.error(e)
                raise AppException(f'file: {filePath} is incorrect!') from e
        else:
            err_msg = 'No sufficient access to the file.'
            app_logger.error(err_msg)
            raise AppException(err_msg)
    else:
        err_msg = 'The file or directory does not exist!'
        app_logger.error(err_msg)
        raise AppException(err_msg)


class AppException(Exception):
    pass



import logging
import platform
from os import makedirs, path
from tempfile import gettempdir
from logging.handlers import TimedRotatingFileHandler


class AppDefaultFormater(logging.Formatter):
    def formatMessage(self, record: logging.LogRecord) -> str:
        seperator = " " * (8 - len(record.levelname))
        record.__dict__["levelprefix"] = record.levelname + ":" + seperator
        return super().formatMessage(record)


class AppTimedRotatingFileHandler(TimedRotatingFileHandler):
    def __init__(self,
                 filename,
                 when,
                 interval=1,
                 backupCount=0,
                 encoding=None,
                 delay=False,
                 utc=False,
                 atTime=None):
        if platform.system() == 'Windows':
            self.__logs_dir = f'{gettempdir()}\\{__package__}\\logs'
            self.__filename = f'{self.__logs_dir}\\{filename}'
        elif platform.system() == 'Linux':
            self.__logs_dir = f'{gettempdir()}/{__package__}/logs'
            self.__filename = f'{self.__logs_dir}/{filename}'

        if not path.exists(self.__logs_dir):
            # if the logs directory is not present
            # then create it.
            makedirs(self.__logs_dir)
        super(AppTimedRotatingFileHandler,
              self).__init__(filename=self.__filename,
                             when=when,
                             interval=interval,
                             backupCount=backupCount,
                             encoding=encoding,
                             delay=delay,
                             utc=utc,
                             atTime=atTime)



from csv import DictReader

import pytest
from chispa.schema_comparer import assert_schema_equality
from pyspark.sql.types import StringType, StructField, StructType

from userstory8020.app import DataFrameManipulation


@pytest.fixture
def df_manipulation(spark):
    print("dsfsdf")
    test_filename_client = 'source_data/dataset_one.csv'
    test_filename_findetails = 'source_data/dataset_two.csv'
    dfm = DataFrameManipulation()
    # open file in read mode
    with open(test_filename_client, 'r') as f:
        dict_reader = DictReader(f)
        list_of_dict = list(dict_reader)
        dfm.clients = spark.createDataFrame(list_of_dict)

    with open(test_filename_findetails, 'r') as f:
        dict_reader = DictReader(f)
        list_of_dict = list(dict_reader)
        dfm.finDetails = spark.createDataFrame(list_of_dict)
    return dfm


def test_counts(df_manipulation):
    assert df_manipulation.clients.count() == 1000
    assert df_manipulation.finDetails.count() == 1000

    df_manipulation.filter_rows(filterConditions={"country": ["United Kingdom",
                                                              "Netherlands"]})

    assert df_manipulation.clients.count() == 100


def test_df_schema(df_manipulation, spark):

    df_manipulation.select_columns(colsList=['email', 'country'],
                                   colsMap={"btc_a": "bitcoin_address",
                                            "id": "client_identifier",
                                            "cc_t": "credit_card_type"})

    expected_schema = StructType([StructField('client_identifier', StringType()),
                                  StructField('country', StringType()),
                                  StructField('email', StringType()),
                                  StructField('bitcoin_address', StringType()),
                                  StructField('credit_card_type', StringType())])
    result_schema = df_manipulation.clients.schema
    assert_schema_equality(expected_schema, result_schema)



from logging.config import dictConfig
from os import path

from yaml import safe_load

script_dir = path.dirname(__file__)
with open(path.join(script_dir, 'logging_conf.yml')) as file:
    loaded_config = safe_load(file)
    dictConfig(loaded_config)
